@@ -34,8 +34,13 @@
 
 #ifdef _MSC_VER
 #include <basetsd.h>
+#if (_MSC_VER <= 1200)
+typedef   signed short  int16_t;
+typedef unsigned short uint16_t;
+#else
 typedef  INT16  int16_t;
 typedef UINT16 uint16_t;
+#endif
 typedef  INT32  int32_t;
 typedef UINT32 uint32_t;
 typedef UINT64 uint64_t;
@@ -218,7 +223,7 @@ void mpi_swap( mpi *X, mpi *Y );
  */
 int mpi_lset( mpi *X, t_sint z );
 
-/*
+/**
  * \brief          Get a specific bit from X
  *
  * \param X        MPI to use
@@ -228,7 +233,7 @@ int mpi_lset( mpi *X, t_sint z );
  */
 int mpi_get_bit( const mpi *X, size_t pos );
 
-/*
+/**
  * \brief          Set a bit of X to a specific value of 0 or 1
  *
  * \note           Will grow X if necessary to set a bit to 1 in a not yet
