@@ -25,6 +25,9 @@ void shutdown_monitor_module(t_config *config);
 
 int  monitor_server_start(void);
 int  monitor_server_stop(void);
+#ifdef ENABLE_LOADCHECK
+int  monitor_high_server_load(double load);
+#endif
 int  monitor_stats(t_config *config, time_t now);
 int  monitor_request(t_session *session);
 
